# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..shared.artist_object import ArtistObject

__all__ = ["FollowingBulkRetrieveResponse", "Artists", "ArtistsCursors"]


class ArtistsCursors(BaseModel):
    """The cursors used to find the next set of items."""

    after: Optional[str] = None
    """The cursor to use as key to find the next page of items."""

    before: Optional[str] = None
    """The cursor to use as key to find the previous page of items."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Artists(BaseModel):
    cursors: Optional[ArtistsCursors] = None
    """The cursors used to find the next set of items."""

    href: Optional[str] = None
    """A link to the Web API endpoint returning the full result of the request."""

    items: Optional[List[ArtistObject]] = None

    limit: Optional[int] = None
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    total: Optional[int] = None
    """The total number of items available to return."""


class FollowingBulkRetrieveResponse(BaseModel):
    artists: Artists



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ImageObject"]


class ImageObject(BaseModel):
    height: Optional[int] = None
    """The image height in pixels."""

    url: str
    """The source URL of the image."""

    width: Optional[int] = None
    """The image width in pixels."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.image_object import ImageObject
from .shared.external_url_object import ExternalURLObject
from .shared.resume_point_object import ResumePointObject
from .shared.chapter_restriction_object import ChapterRestrictionObject

__all__ = ["SimplifiedChapterObject"]


class SimplifiedChapterObject(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    chapter.
    """

    audio_preview_url: Optional[str] = None
    """A URL to a 30 second preview (MP3 format) of the chapter.

    `null` if not available.
    """

    chapter_number: int
    """The number of the chapter"""

    description: str
    """A description of the chapter.

    HTML tags are stripped away from this field, use `html_description` field in
    case HTML tags are needed.
    """

    duration_ms: int
    """The chapter length in milliseconds."""

    explicit: bool
    """
    Whether or not the chapter has explicit content (true = yes it does; false = no
    it does not OR unknown).
    """

    external_urls: ExternalURLObject
    """External URLs for this chapter."""

    href: str
    """A link to the Web API endpoint providing full details of the chapter."""

    html_description: str
    """A description of the chapter. This field may contain HTML tags."""

    images: List[ImageObject]
    """The cover art for the chapter in various sizes, widest first."""

    is_playable: bool
    """True if the chapter is playable in the given market. Otherwise false."""

    languages: List[str]
    """
    A list of the languages used in the chapter, identified by their
    [ISO 639-1](https://en.wikipedia.org/wiki/ISO_639) code.
    """

    name: str
    """The name of the chapter."""

    release_date: str
    """The date the chapter was first released, for example `"1981-12-15"`.

    Depending on the precision, it might be shown as `"1981"` or `"1981-12"`.
    """

    release_date_precision: Literal["year", "month", "day"]
    """The precision with which `release_date` value is known."""

    type: Literal["episode"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    chapter.
    """

    available_markets: Optional[List[str]] = None
    """
    A list of the countries in which the chapter can be played, identified by their
    [ISO 3166-1 alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    restrictions: Optional[ChapterRestrictionObject] = None
    """Included in the response when a content restriction is applied."""

    resume_point: Optional[ResumePointObject] = None
    """The user's most recent position in the chapter.

    Set if the supplied access token is a user token and has the scope
    'user-read-playback-position'.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PlayerSkipNextParams"]


class PlayerSkipNextParams(TypedDict, total=False):
    device_id: str
    """The id of the device this command is targeting.

    If not supplied, the user's currently active device is the target.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import (
    SimplifiedChapterObject,
    AudiobookRetrieveResponse,
    AudiobookBulkRetrieveResponse,
)
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAudiobooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Spotted) -> None:
        audiobook = client.audiobooks.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )
        assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Spotted) -> None:
        audiobook = client.audiobooks.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
            market="ES",
        )
        assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Spotted) -> None:
        response = client.audiobooks.with_raw_response.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Spotted) -> None:
        with client.audiobooks.with_streaming_response.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = response.parse()
            assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.audiobooks.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            audiobook = client.audiobooks.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
            )

        assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_bulk_retrieve_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            audiobook = client.audiobooks.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
                market="ES",
            )

        assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.audiobooks.with_raw_response.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.audiobooks.with_streaming_response.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                audiobook = response.parse()
                assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_chapters(self, client: Spotted) -> None:
        audiobook = client.audiobooks.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )
        assert_matches_type(SyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_chapters_with_all_params(self, client: Spotted) -> None:
        audiobook = client.audiobooks.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
            limit=10,
            market="ES",
            offset=5,
        )
        assert_matches_type(SyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_chapters(self, client: Spotted) -> None:
        response = client.audiobooks.with_raw_response.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert_matches_type(SyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_chapters(self, client: Spotted) -> None:
        with client.audiobooks.with_streaming_response.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = response.parse()
            assert_matches_type(SyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_chapters(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.audiobooks.with_raw_response.list_chapters(
                id="",
            )


class TestAsyncAudiobooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.audiobooks.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )
        assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.audiobooks.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
            market="ES",
        )
        assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.audiobooks.with_raw_response.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.audiobooks.with_streaming_response.retrieve(
            id="7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = await response.parse()
            assert_matches_type(AudiobookRetrieveResponse, audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.audiobooks.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            audiobook = await async_client.audiobooks.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
            )

        assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            audiobook = await async_client.audiobooks.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
                market="ES",
            )

        assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.audiobooks.with_raw_response.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.audiobooks.with_streaming_response.bulk_retrieve(
                ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                audiobook = await response.parse()
                assert_matches_type(AudiobookBulkRetrieveResponse, audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_chapters(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.audiobooks.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )
        assert_matches_type(AsyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_chapters_with_all_params(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.audiobooks.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
            limit=10,
            market="ES",
            offset=5,
        )
        assert_matches_type(AsyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_chapters(self, async_client: AsyncSpotted) -> None:
        response = await async_client.audiobooks.with_raw_response.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert_matches_type(AsyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_chapters(self, async_client: AsyncSpotted) -> None:
        async with async_client.audiobooks.with_streaming_response.list_chapters(
            id="7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = await response.parse()
            assert_matches_type(AsyncCursorURLPage[SimplifiedChapterObject], audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_chapters(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.audiobooks.with_raw_response.list_chapters(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .images import (
    ImagesResource,
    AsyncImagesResource,
    ImagesResourceWithRawResponse,
    AsyncImagesResourceWithRawResponse,
    ImagesResourceWithStreamingResponse,
    AsyncImagesResourceWithStreamingResponse,
)
from .tracks import (
    TracksResource,
    AsyncTracksResource,
    TracksResourceWithRawResponse,
    AsyncTracksResourceWithRawResponse,
    TracksResourceWithStreamingResponse,
    AsyncTracksResourceWithStreamingResponse,
)
from .followers import (
    FollowersResource,
    AsyncFollowersResource,
    FollowersResourceWithRawResponse,
    AsyncFollowersResourceWithRawResponse,
    FollowersResourceWithStreamingResponse,
    AsyncFollowersResourceWithStreamingResponse,
)
from .playlists import (
    PlaylistsResource,
    AsyncPlaylistsResource,
    PlaylistsResourceWithRawResponse,
    AsyncPlaylistsResourceWithRawResponse,
    PlaylistsResourceWithStreamingResponse,
    AsyncPlaylistsResourceWithStreamingResponse,
)

__all__ = [
    "TracksResource",
    "AsyncTracksResource",
    "TracksResourceWithRawResponse",
    "AsyncTracksResourceWithRawResponse",
    "TracksResourceWithStreamingResponse",
    "AsyncTracksResourceWithStreamingResponse",
    "FollowersResource",
    "AsyncFollowersResource",
    "FollowersResourceWithRawResponse",
    "AsyncFollowersResourceWithRawResponse",
    "FollowersResourceWithStreamingResponse",
    "AsyncFollowersResourceWithStreamingResponse",
    "ImagesResource",
    "AsyncImagesResource",
    "ImagesResourceWithRawResponse",
    "AsyncImagesResourceWithRawResponse",
    "ImagesResourceWithStreamingResponse",
    "AsyncImagesResourceWithStreamingResponse",
    "PlaylistsResource",
    "AsyncPlaylistsResource",
    "PlaylistsResourceWithRawResponse",
    "AsyncPlaylistsResourceWithRawResponse",
    "PlaylistsResourceWithStreamingResponse",
    "AsyncPlaylistsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import EpisodeBulkRetrieveResponse
from spotted.types.shared import EpisodeObject

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEpisodes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Spotted) -> None:
        episode = client.episodes.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
        )
        assert_matches_type(EpisodeObject, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Spotted) -> None:
        episode = client.episodes.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
            market="ES",
        )
        assert_matches_type(EpisodeObject, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Spotted) -> None:
        response = client.episodes.with_raw_response.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        episode = response.parse()
        assert_matches_type(EpisodeObject, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Spotted) -> None:
        with client.episodes.with_streaming_response.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            episode = response.parse()
            assert_matches_type(EpisodeObject, episode, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.episodes.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            episode = client.episodes.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
            )

        assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_bulk_retrieve_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            episode = client.episodes.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
                market="ES",
            )

        assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.episodes.with_raw_response.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        episode = response.parse()
        assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.episodes.with_streaming_response.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                episode = response.parse()
                assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncEpisodes:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSpotted) -> None:
        episode = await async_client.episodes.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
        )
        assert_matches_type(EpisodeObject, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        episode = await async_client.episodes.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
            market="ES",
        )
        assert_matches_type(EpisodeObject, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.episodes.with_raw_response.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        episode = await response.parse()
        assert_matches_type(EpisodeObject, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.episodes.with_streaming_response.retrieve(
            id="512ojhOuo1ktJprKbVcKyQ",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            episode = await response.parse()
            assert_matches_type(EpisodeObject, episode, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.episodes.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            episode = await async_client.episodes.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
            )

        assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            episode = await async_client.episodes.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
                market="ES",
            )

        assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.episodes.with_raw_response.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        episode = await response.parse()
        assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.episodes.with_streaming_response.bulk_retrieve(
                ids="77o6BIVlYM3msb4MMIL1jH,0Q86acNRm6V9GYx55SXKwf",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                episode = await response.parse()
                assert_matches_type(EpisodeBulkRetrieveResponse, episode, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import MarketListResponse

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMarkets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            market = client.markets.list()

        assert_matches_type(MarketListResponse, market, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.markets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        market = response.parse()
        assert_matches_type(MarketListResponse, market, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.markets.with_streaming_response.list() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                market = response.parse()
                assert_matches_type(MarketListResponse, market, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMarkets:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            market = await async_client.markets.list()

        assert_matches_type(MarketListResponse, market, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.markets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        market = await response.parse()
        assert_matches_type(MarketListResponse, market, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.markets.with_streaming_response.list() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                market = await response.parse()
                assert_matches_type(MarketListResponse, market, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel
from ..shared.show_base import ShowBase

__all__ = ["ShowListResponse"]


class ShowListResponse(BaseModel):
    added_at: Optional[datetime] = None
    """
    The date and time the show was saved. Timestamps are returned in ISO 8601 format
    as Coordinated Universal Time (UTC) with a zero offset: YYYY-MM-DDTHH:MM:SSZ. If
    the time is imprecise (for example, the date/time of an album release), an
    additional field indicates the precision; see for example, release_date in an
    album object.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    show: Optional[ShowBase] = None
    """Information about the show."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PlaylistListParams"]


class PlaylistListParams(TypedDict, total=False):
    limit: int
    """The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50."""

    offset: int
    """'The index of the first playlist to return.

    Default: 0 (the first object). Maximum offset: 100.000\\.. Use with `limit` to get
    the next set of playlists.'
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .queue_add_params import QueueAddParams as QueueAddParams
from .queue_get_response import QueueGetResponse as QueueGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel
from ..shared.track_object import TrackObject

__all__ = ["TrackListResponse"]


class TrackListResponse(BaseModel):
    added_at: Optional[datetime] = None
    """
    The date and time the track was saved. Timestamps are returned in ISO 8601
    format as Coordinated Universal Time (UTC) with a zero offset:
    YYYY-MM-DDTHH:MM:SSZ. If the time is imprecise (for example, the date/time of an
    album release), an additional field indicates the precision; see for example,
    release_date in an album object.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    track: Optional[TrackObject] = None
    """Information about the track."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from .playlists import (
    PlaylistsResource,
    AsyncPlaylistsResource,
    PlaylistsResourceWithRawResponse,
    AsyncPlaylistsResourceWithRawResponse,
    PlaylistsResourceWithStreamingResponse,
    AsyncPlaylistsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.user_retrieve_profile_response import UserRetrieveProfileResponse

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    @cached_property
    def playlists(self) -> PlaylistsResource:
        return PlaylistsResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def retrieve_profile(
        self,
        user_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserRetrieveProfileResponse:
        """
        Get public profile information about a Spotify user.

        Args:
          user_id: The user's [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            f"/users/{user_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserRetrieveProfileResponse,
        )


class AsyncUsersResource(AsyncAPIResource):
    @cached_property
    def playlists(self) -> AsyncPlaylistsResource:
        return AsyncPlaylistsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def retrieve_profile(
        self,
        user_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserRetrieveProfileResponse:
        """
        Get public profile information about a Spotify user.

        Args:
          user_id: The user's [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            f"/users/{user_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserRetrieveProfileResponse,
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve_profile = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                users.retrieve_profile,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def playlists(self) -> PlaylistsResourceWithRawResponse:
        return PlaylistsResourceWithRawResponse(self._users.playlists)


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve_profile = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                users.retrieve_profile,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def playlists(self) -> AsyncPlaylistsResourceWithRawResponse:
        return AsyncPlaylistsResourceWithRawResponse(self._users.playlists)


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve_profile = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                users.retrieve_profile,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def playlists(self) -> PlaylistsResourceWithStreamingResponse:
        return PlaylistsResourceWithStreamingResponse(self._users.playlists)


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve_profile = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                users.retrieve_profile,  # pyright: ignore[reportDeprecated],
            )
        )

    @cached_property
    def playlists(self) -> AsyncPlaylistsResourceWithStreamingResponse:
        return AsyncPlaylistsResourceWithStreamingResponse(self._users.playlists)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage
from spotted.types.shared import PlaylistTrackObject
from spotted.types.playlists import (
    TrackAddResponse,
    TrackRemoveResponse,
    TrackUpdateResponse,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTracks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(TrackUpdateResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                insert_before=3,
                published=True,
                range_length=2,
                range_start=1,
                snapshot_id="snapshot_id",
                uris=["string"],
            )

        assert_matches_type(TrackUpdateResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.tracks.with_raw_response.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = response.parse()
        assert_matches_type(TrackUpdateResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.tracks.with_streaming_response.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = response.parse()
                assert_matches_type(TrackUpdateResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.tracks.with_raw_response.update(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(SyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                additional_types="additional_types",
                fields="items(added_by.id,track(name,href,album(name,href)))",
                limit=10,
                market="ES",
                offset=5,
            )

        assert_matches_type(SyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.tracks.with_raw_response.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = response.parse()
        assert_matches_type(SyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.tracks.with_streaming_response.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = response.parse()
                assert_matches_type(SyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.tracks.with_raw_response.list(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_add(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(TrackAddResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_add_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                position=0,
                published=True,
                uris=["string"],
            )

        assert_matches_type(TrackAddResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_add(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.tracks.with_raw_response.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = response.parse()
        assert_matches_type(TrackAddResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_add(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.tracks.with_streaming_response.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = response.parse()
                assert_matches_type(TrackAddResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_add(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.tracks.with_raw_response.add(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{}],
            )

        assert_matches_type(TrackRemoveResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = client.playlists.tracks.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{"uri": "uri"}],
                published=True,
                snapshot_id="snapshot_id",
            )

        assert_matches_type(TrackRemoveResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.tracks.with_raw_response.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{}],
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = response.parse()
        assert_matches_type(TrackRemoveResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.tracks.with_streaming_response.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{}],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = response.parse()
                assert_matches_type(TrackRemoveResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.tracks.with_raw_response.remove(
                    playlist_id="",
                    tracks=[{}],
                )


class TestAsyncTracks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(TrackUpdateResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                insert_before=3,
                published=True,
                range_length=2,
                range_start=1,
                snapshot_id="snapshot_id",
                uris=["string"],
            )

        assert_matches_type(TrackUpdateResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.tracks.with_raw_response.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = await response.parse()
        assert_matches_type(TrackUpdateResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.tracks.with_streaming_response.update(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = await response.parse()
                assert_matches_type(TrackUpdateResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.tracks.with_raw_response.update(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(AsyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                additional_types="additional_types",
                fields="items(added_by.id,track(name,href,album(name,href)))",
                limit=10,
                market="ES",
                offset=5,
            )

        assert_matches_type(AsyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.tracks.with_raw_response.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = await response.parse()
        assert_matches_type(AsyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.tracks.with_streaming_response.list(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = await response.parse()
                assert_matches_type(AsyncCursorURLPage[PlaylistTrackObject], track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.tracks.with_raw_response.list(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_add(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(TrackAddResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_add_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                position=0,
                published=True,
                uris=["string"],
            )

        assert_matches_type(TrackAddResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_add(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.tracks.with_raw_response.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = await response.parse()
        assert_matches_type(TrackAddResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_add(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.tracks.with_streaming_response.add(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = await response.parse()
                assert_matches_type(TrackAddResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_add(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.tracks.with_raw_response.add(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{}],
            )

        assert_matches_type(TrackRemoveResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            track = await async_client.playlists.tracks.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{"uri": "uri"}],
                published=True,
                snapshot_id="snapshot_id",
            )

        assert_matches_type(TrackRemoveResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.tracks.with_raw_response.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{}],
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        track = await response.parse()
        assert_matches_type(TrackRemoveResponse, track, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.tracks.with_streaming_response.remove(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                tracks=[{}],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                track = await response.parse()
                assert_matches_type(TrackRemoveResponse, track, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.tracks.with_raw_response.remove(
                    playlist_id="",
                    tracks=[{}],
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import (
    ArtistTopTracksResponse,
    ArtistListAlbumsResponse,
    ArtistBulkRetrieveResponse,
    ArtistListRelatedArtistsResponse,
)
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage
from spotted.types.shared import ArtistObject

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestArtists:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Spotted) -> None:
        artist = client.artists.retrieve(
            "0TnOYISbd1XYRBk9myaseg",
        )
        assert_matches_type(ArtistObject, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Spotted) -> None:
        response = client.artists.with_raw_response.retrieve(
            "0TnOYISbd1XYRBk9myaseg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = response.parse()
        assert_matches_type(ArtistObject, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Spotted) -> None:
        with client.artists.with_streaming_response.retrieve(
            "0TnOYISbd1XYRBk9myaseg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            artist = response.parse()
            assert_matches_type(ArtistObject, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.artists.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = client.artists.bulk_retrieve(
                ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            )

        assert_matches_type(ArtistBulkRetrieveResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.artists.with_raw_response.bulk_retrieve(
                ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = response.parse()
        assert_matches_type(ArtistBulkRetrieveResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_bulk_retrieve(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.artists.with_streaming_response.bulk_retrieve(
                ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                artist = response.parse()
                assert_matches_type(ArtistBulkRetrieveResponse, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_albums(self, client: Spotted) -> None:
        artist = client.artists.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
        )
        assert_matches_type(SyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_albums_with_all_params(self, client: Spotted) -> None:
        artist = client.artists.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
            include_groups="single,appears_on",
            limit=5,
            market="ES",
            offset=5,
        )
        assert_matches_type(SyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_albums(self, client: Spotted) -> None:
        response = client.artists.with_raw_response.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = response.parse()
        assert_matches_type(SyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_albums(self, client: Spotted) -> None:
        with client.artists.with_streaming_response.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            artist = response.parse()
            assert_matches_type(SyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_albums(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.artists.with_raw_response.list_albums(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_related_artists(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = client.artists.list_related_artists(
                "0TnOYISbd1XYRBk9myaseg",
            )

        assert_matches_type(ArtistListRelatedArtistsResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_related_artists(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.artists.with_raw_response.list_related_artists(
                "0TnOYISbd1XYRBk9myaseg",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = response.parse()
        assert_matches_type(ArtistListRelatedArtistsResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_related_artists(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.artists.with_streaming_response.list_related_artists(
                "0TnOYISbd1XYRBk9myaseg",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                artist = response.parse()
                assert_matches_type(ArtistListRelatedArtistsResponse, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_related_artists(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
                client.artists.with_raw_response.list_related_artists(
                    "",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_top_tracks(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = client.artists.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
            )

        assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_top_tracks_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = client.artists.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
                market="ES",
            )

        assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_top_tracks(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.artists.with_raw_response.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = response.parse()
        assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_top_tracks(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.artists.with_streaming_response.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                artist = response.parse()
                assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_top_tracks(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
                client.artists.with_raw_response.top_tracks(
                    id="",
                )


class TestAsyncArtists:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSpotted) -> None:
        artist = await async_client.artists.retrieve(
            "0TnOYISbd1XYRBk9myaseg",
        )
        assert_matches_type(ArtistObject, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.artists.with_raw_response.retrieve(
            "0TnOYISbd1XYRBk9myaseg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = await response.parse()
        assert_matches_type(ArtistObject, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.artists.with_streaming_response.retrieve(
            "0TnOYISbd1XYRBk9myaseg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            artist = await response.parse()
            assert_matches_type(ArtistObject, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.artists.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = await async_client.artists.bulk_retrieve(
                ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            )

        assert_matches_type(ArtistBulkRetrieveResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.artists.with_raw_response.bulk_retrieve(
                ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = await response.parse()
        assert_matches_type(ArtistBulkRetrieveResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.artists.with_streaming_response.bulk_retrieve(
                ids="2CIMQHirSU0MQqyYHq0eOx,57dN52uHvrHOxijzpIgu3E,1vCWHaC5f2uS3yhpwWbIA6",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                artist = await response.parse()
                assert_matches_type(ArtistBulkRetrieveResponse, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_albums(self, async_client: AsyncSpotted) -> None:
        artist = await async_client.artists.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
        )
        assert_matches_type(AsyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_albums_with_all_params(self, async_client: AsyncSpotted) -> None:
        artist = await async_client.artists.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
            include_groups="single,appears_on",
            limit=5,
            market="ES",
            offset=5,
        )
        assert_matches_type(AsyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_albums(self, async_client: AsyncSpotted) -> None:
        response = await async_client.artists.with_raw_response.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = await response.parse()
        assert_matches_type(AsyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_albums(self, async_client: AsyncSpotted) -> None:
        async with async_client.artists.with_streaming_response.list_albums(
            id="0TnOYISbd1XYRBk9myaseg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            artist = await response.parse()
            assert_matches_type(AsyncCursorURLPage[ArtistListAlbumsResponse], artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_albums(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.artists.with_raw_response.list_albums(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_related_artists(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = await async_client.artists.list_related_artists(
                "0TnOYISbd1XYRBk9myaseg",
            )

        assert_matches_type(ArtistListRelatedArtistsResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_related_artists(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.artists.with_raw_response.list_related_artists(
                "0TnOYISbd1XYRBk9myaseg",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = await response.parse()
        assert_matches_type(ArtistListRelatedArtistsResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_related_artists(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.artists.with_streaming_response.list_related_artists(
                "0TnOYISbd1XYRBk9myaseg",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                artist = await response.parse()
                assert_matches_type(ArtistListRelatedArtistsResponse, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_related_artists(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
                await async_client.artists.with_raw_response.list_related_artists(
                    "",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_top_tracks(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = await async_client.artists.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
            )

        assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_top_tracks_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            artist = await async_client.artists.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
                market="ES",
            )

        assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_top_tracks(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.artists.with_raw_response.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        artist = await response.parse()
        assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_top_tracks(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.artists.with_streaming_response.top_tracks(
                id="0TnOYISbd1XYRBk9myaseg",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                artist = await response.parse()
                assert_matches_type(ArtistTopTracksResponse, artist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_top_tracks(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
                await async_client.artists.with_raw_response.top_tracks(
                    id="",
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.show_base import ShowBase
from .shared.image_object import ImageObject
from .shared.track_object import TrackObject
from .shared.artist_object import ArtistObject
from .shared.audiobook_base import AudiobookBase
from .shared.external_url_object import ExternalURLObject
from .shared.paging_playlist_object import PagingPlaylistObject
from .shared.album_restriction_object import AlbumRestrictionObject
from .shared.simplified_artist_object import SimplifiedArtistObject
from .shared.simplified_episode_object import SimplifiedEpisodeObject

__all__ = ["SearchQueryResponse", "Albums", "AlbumsItem", "Artists", "Audiobooks", "Episodes", "Shows", "Tracks"]


class AlbumsItem(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    album_type: Literal["album", "single", "compilation"]
    """The type of the album."""

    artists: List[SimplifiedArtistObject]
    """The artists of the album.

    Each artist object includes a link in `href` to more detailed information about
    the artist.
    """

    available_markets: List[str]
    """
    The markets in which the album is available:
    [ISO 3166-1 alpha-2 country codes](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    _**NOTE**: an album is considered available in a market when at least 1 of its
    tracks is available in that market._
    """

    external_urls: ExternalURLObject
    """Known external URLs for this album."""

    href: str
    """A link to the Web API endpoint providing full details of the album."""

    images: List[ImageObject]
    """The cover art for the album in various sizes, widest first."""

    name: str
    """The name of the album.

    In case of an album takedown, the value may be an empty string.
    """

    release_date: str
    """The date the album was first released."""

    release_date_precision: Literal["year", "month", "day"]
    """The precision with which `release_date` value is known."""

    total_tracks: int
    """The number of tracks in the album."""

    type: Literal["album"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    restrictions: Optional[AlbumRestrictionObject] = None
    """Included in the response when a content restriction is applied."""


class Albums(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[AlbumsItem]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Artists(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[ArtistObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Audiobooks(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[AudiobookBase]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Episodes(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[SimplifiedEpisodeObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Shows(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[ShowBase]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Tracks(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[TrackObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class SearchQueryResponse(BaseModel):
    albums: Optional[Albums] = None

    artists: Optional[Artists] = None

    audiobooks: Optional[Audiobooks] = None

    episodes: Optional[Episodes] = None

    playlists: Optional[PagingPlaylistObject] = None

    shows: Optional[Shows] = None

    tracks: Optional[Tracks] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .shared.episode_object import EpisodeObject

__all__ = ["EpisodeBulkRetrieveResponse"]


class EpisodeBulkRetrieveResponse(BaseModel):
    episodes: List[EpisodeObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os

import httpx

from ..._files import read_file_content, async_read_file_content
from ..._types import (
    Body,
    Query,
    Headers,
    NotGiven,
    BinaryTypes,
    FileContent,
    AsyncBinaryTypes,
    not_given,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.playlists.image_list_response import ImageListResponse

__all__ = ["ImagesResource", "AsyncImagesResource"]


class ImagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ImagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return ImagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ImagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return ImagesResourceWithStreamingResponse(self)

    def update(
        self,
        playlist_id: str,
        body: FileContent | BinaryTypes,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Replace the image used to represent a specific playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          body: Base64 encoded JPEG image data, maximum payload size is 256 KB.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "application/binary", **(extra_headers or {})}
        extra_headers = {"Content-Type": "image/jpeg", **(extra_headers or {})}
        return self._put(
            f"/playlists/{playlist_id}/images",
            content=read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def list(
        self,
        playlist_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImageListResponse:
        """
        Get the current image associated with a specific playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._get(
            f"/playlists/{playlist_id}/images",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImageListResponse,
        )


class AsyncImagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncImagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncImagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncImagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncImagesResourceWithStreamingResponse(self)

    async def update(
        self,
        playlist_id: str,
        body: FileContent | AsyncBinaryTypes,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Replace the image used to represent a specific playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          body: Base64 encoded JPEG image data, maximum payload size is 256 KB.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "application/binary", **(extra_headers or {})}
        extra_headers = {"Content-Type": "image/jpeg", **(extra_headers or {})}
        return await self._put(
            f"/playlists/{playlist_id}/images",
            content=await async_read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def list(
        self,
        playlist_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImageListResponse:
        """
        Get the current image associated with a specific playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return await self._get(
            f"/playlists/{playlist_id}/images",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImageListResponse,
        )


class ImagesResourceWithRawResponse:
    def __init__(self, images: ImagesResource) -> None:
        self._images = images

        self.update = to_custom_raw_response_wrapper(
            images.update,
            BinaryAPIResponse,
        )
        self.list = to_raw_response_wrapper(
            images.list,
        )


class AsyncImagesResourceWithRawResponse:
    def __init__(self, images: AsyncImagesResource) -> None:
        self._images = images

        self.update = async_to_custom_raw_response_wrapper(
            images.update,
            AsyncBinaryAPIResponse,
        )
        self.list = async_to_raw_response_wrapper(
            images.list,
        )


class ImagesResourceWithStreamingResponse:
    def __init__(self, images: ImagesResource) -> None:
        self._images = images

        self.update = to_custom_streamed_response_wrapper(
            images.update,
            StreamedBinaryAPIResponse,
        )
        self.list = to_streamed_response_wrapper(
            images.list,
        )


class AsyncImagesResourceWithStreamingResponse:
    def __init__(self, images: AsyncImagesResource) -> None:
        self._images = images

        self.update = async_to_custom_streamed_response_wrapper(
            images.update,
            AsyncStreamedBinaryAPIResponse,
        )
        self.list = async_to_streamed_response_wrapper(
            images.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FollowerCheckParams"]


class FollowerCheckParams(TypedDict, total=False):
    ids: str
    """
    **Deprecated** A single item list containing current user's
    [Spotify Username](/documentation/web-api/concepts/spotify-uris-ids). Maximum: 1
    id.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PlaylistRetrieveParams"]


class PlaylistRetrieveParams(TypedDict, total=False):
    additional_types: str
    """
    A comma-separated list of item types that your client supports besides the
    default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
    This parameter was introduced to allow existing clients to maintain their
    current behaviour and might be deprecated in the future._<br/> In addition to
    providing this parameter, make sure that your client properly handles cases of
    new types in the future by checking against the `type` field of each object.
    """

    fields: str
    """Filters for the query: a comma-separated list of the fields to return.

    If omitted, all fields are returned. For example, to get just the playlist''s
    description and URI: `fields=description,uri`. A dot separator can be used to
    specify non-reoccurring fields, while parentheses can be used to specify
    reoccurring fields within objects. For example, to get just the added date and
    user ID of the adder: `fields=tracks.items(added_at,added_by.id)`. Use multiple
    parentheses to drill down into nested objects, for example:
    `fields=tracks.items(track(name,href,album(name,href)))`. Fields can be excluded
    by prefixing them with an exclamation mark, for example:
    `fields=tracks.items(track(name,href,album(!name,href)))`
    """

    market: str
    """
    An
    [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    If a country code is specified, only content that is available in that market
    will be returned.<br/> If a valid user access token is specified in the request
    header, the country associated with the user account will take priority over
    this parameter.<br/> _**Note**: If neither market or user country are provided,
    the content is considered unavailable for the client._<br/> Users can view the
    country that is associated with their account in the
    [account settings](https://www.spotify.com/account/overview/).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.image_object import ImageObject
from ..shared.copyright_object import CopyrightObject
from ..shared.external_id_object import ExternalIDObject
from ..shared.external_url_object import ExternalURLObject
from ..shared.simplified_track_object import SimplifiedTrackObject
from ..shared.album_restriction_object import AlbumRestrictionObject
from ..shared.simplified_artist_object import SimplifiedArtistObject

__all__ = ["AlbumListResponse", "Album", "AlbumTracks"]


class AlbumTracks(BaseModel):
    """The tracks of the album."""

    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[SimplifiedTrackObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Album(BaseModel):
    """Information about the album."""

    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    album_type: Literal["album", "single", "compilation"]
    """The type of the album."""

    available_markets: List[str]
    """
    The markets in which the album is available:
    [ISO 3166-1 alpha-2 country codes](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    _**NOTE**: an album is considered available in a market when at least 1 of its
    tracks is available in that market._
    """

    external_urls: ExternalURLObject
    """Known external URLs for this album."""

    href: str
    """A link to the Web API endpoint providing full details of the album."""

    images: List[ImageObject]
    """The cover art for the album in various sizes, widest first."""

    name: str
    """The name of the album.

    In case of an album takedown, the value may be an empty string.
    """

    release_date: str
    """The date the album was first released."""

    release_date_precision: Literal["year", "month", "day"]
    """The precision with which `release_date` value is known."""

    total_tracks: int
    """The number of tracks in the album."""

    type: Literal["album"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    artists: Optional[List[SimplifiedArtistObject]] = None
    """The artists of the album.

    Each artist object includes a link in `href` to more detailed information about
    the artist.
    """

    copyrights: Optional[List[CopyrightObject]] = None
    """The copyright statements of the album."""

    external_ids: Optional[ExternalIDObject] = None
    """Known external IDs for the album."""

    genres: Optional[List[str]] = None
    """**Deprecated** The array is always empty."""

    label: Optional[str] = None
    """The label associated with the album."""

    popularity: Optional[int] = None
    """The popularity of the album.

    The value will be between 0 and 100, with 100 being the most popular.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    restrictions: Optional[AlbumRestrictionObject] = None
    """Included in the response when a content restriction is applied."""

    tracks: Optional[AlbumTracks] = None
    """The tracks of the album."""


class AlbumListResponse(BaseModel):
    added_at: Optional[datetime] = None
    """
    The date and time the album was saved Timestamps are returned in ISO 8601 format
    as Coordinated Universal Time (UTC) with a zero offset: YYYY-MM-DDTHH:MM:SSZ. If
    the time is imprecise (for example, the date/time of an album release), an
    additional field indicates the precision; see for example, release_date in an
    album object.
    """

    album: Optional[Album] = None
    """Information about the album."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .device_object import DeviceObject
from .context_object import ContextObject
from ..shared.track_object import TrackObject
from ..shared.episode_object import EpisodeObject

__all__ = ["PlayerGetStateResponse", "Actions", "Item"]


class Actions(BaseModel):
    """
    Allows to update the user interface based on which playback actions are available within the current context.
    """

    interrupting_playback: Optional[bool] = None
    """Interrupting playback. Optional field."""

    pausing: Optional[bool] = None
    """Pausing. Optional field."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    resuming: Optional[bool] = None
    """Resuming. Optional field."""

    seeking: Optional[bool] = None
    """Seeking playback location. Optional field."""

    skipping_next: Optional[bool] = None
    """Skipping to the next context. Optional field."""

    skipping_prev: Optional[bool] = None
    """Skipping to the previous context. Optional field."""

    toggling_repeat_context: Optional[bool] = None
    """Toggling repeat context flag. Optional field."""

    toggling_repeat_track: Optional[bool] = None
    """Toggling repeat track flag. Optional field."""

    toggling_shuffle: Optional[bool] = None
    """Toggling shuffle flag. Optional field."""

    transferring_playback: Optional[bool] = None
    """Transfering playback between devices. Optional field."""


Item: TypeAlias = Annotated[Union[TrackObject, EpisodeObject], PropertyInfo(discriminator="type")]


class PlayerGetStateResponse(BaseModel):
    actions: Optional[Actions] = None
    """
    Allows to update the user interface based on which playback actions are
    available within the current context.
    """

    context: Optional[ContextObject] = None
    """A Context Object. Can be `null`."""

    currently_playing_type: Optional[str] = None
    """The object type of the currently playing item.

    Can be one of `track`, `episode`, `ad` or `unknown`.
    """

    device: Optional[DeviceObject] = None
    """The device that is currently active."""

    is_playing: Optional[bool] = None
    """If something is currently playing, return `true`."""

    item: Optional[Item] = None
    """The currently playing track or episode. Can be `null`."""

    progress_ms: Optional[int] = None
    """Progress into the currently playing track or episode. Can be `null`."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    repeat_state: Optional[str] = None
    """off, track, context"""

    shuffle_state: Optional[bool] = None
    """If shuffle is on or off."""

    timestamp: Optional[int] = None
    """
    Unix Millisecond Timestamp when playback state was last changed (play, pause,
    skip, scrub, new song, etc.).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ChapterRestrictionObject"]


class ChapterRestrictionObject(BaseModel):
    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    reason: Optional[str] = None
    """The reason for the restriction. Supported values:

    - `market` - The content item is not available in the given market.
    - `product` - The content item is not available for the user's subscription
      type.
    - `explicit` - The content item is explicit and the user's account is set to not
      play explicit content.
    - `payment_required` - Payment is required to play the content item.

    Additional reasons may be added in the future. **Note**: If you use this field,
    make sure that your application safely handles unknown values.
    """
